@@ -10,6 +10,10 @@ import time
 from concurrent.futures import Future
 from datetime import datetime
 from threading import Event
+try:
+    from queue import Queue
+except ImportError: # py2
+    from Queue import Queue
 
 from decorator import decorator
 from tornado.gen import multi_future
@@ -39,6 +43,7 @@ def check_ready(f, self, *args, **kwargs):
     return f(self, *args, **kwargs)
 
 _metadata_keys = []
+_FOREVER = int(1e8) # a long time because some infinite timeouts are uninterruptible
 
 class AsyncResult(Future):
     """Class for representing results of non-blocking calls.
@@ -115,12 +120,13 @@ class AsyncResult(Future):
         # on completion of my constituents, trigger my own resolution
         self._result_future.add_done_callback(self._resolve_result)
         self._output_future.add_done_callback(self._resolve_output)
+        self.add_done_callback(self._finalize_result)
 
     def __repr__(self):
         if self._ready:
-            return "<%s: finished>"%(self.__class__.__name__)
+            return "<%s: %s:finished>" % (self.__class__.__name__, self._fname)
         else:
-            return "<%s: %s>"%(self.__class__.__name__,self._fname)
+            return "<%s: %s>" % (self.__class__.__name__, self._fname)
     
     def __dir__(self):
         keys = dir(self.__class__)
@@ -217,17 +223,17 @@ class AsyncResult(Future):
                     raise r
             else:
                 results = error.collect_exceptions(results, self._fname)
+            self._success = True
             self.set_result(self._reconstruct_result(results))
         except Exception as e:
-            self.set_exception(e)
             self._success = False
-        else:
-            self._success = True
-        finally:
-            if self.owner:
-                [ self._client.results.pop(mid, None) for mid in self.msg_ids ]
-            self._ready = True
-            self._ready_event.set()
+            self.set_exception(e)
+    
+    def _finalize_result(self,f):
+        if self.owner:
+            [ self._client.results.pop(mid, None) for mid in self.msg_ids ]
+        self._ready = True
+        self._ready_event.set()
 
     def successful(self):
         """Return whether the call completed without raising an exception.
@@ -361,7 +367,14 @@ class AsyncResult(Future):
         except (error.TimeoutError, KeyError):
             raise AttributeError("%r object has no attribute %r"%(
                     self.__class__.__name__, key))
-
+    
+    @staticmethod
+    def _wait_for_child(child, evt, timeout=_FOREVER):
+        """Wait for a child to be done"""
+        evt.clear()
+        child.add_done_callback(lambda f: evt.set())
+        evt.wait(timeout)
+    
     # asynchronous iterator:
     def __iter__(self):
         if self._single_result:
@@ -370,10 +383,12 @@ class AsyncResult(Future):
             rlist = self.get(0)
         except error.TimeoutError:
             # wait for each result individually
-            
+            evt = Event()
             for child in self._children:
-                ar = AsyncResult(self._client, child, self._fname)
-                yield ar.get()
+                self._wait_for_child(child, evt=evt)
+                result = child.result()
+                error.collect_exceptions([result], self._fname)
+                yield result
         else:
             # already done
             for r in rlist:
@@ -656,7 +671,19 @@ class AsyncMapResult(AsyncResult):
         it = self._ordered_iter if self.ordered else self._unordered_iter
         for r in it():
             yield r
-
+    
+    def _yield_child_results(self, child):
+        """Yield results from a child
+        
+        for use in iterator methods
+        """
+        rlist = child.result()
+        if not isinstance(rlist, list):
+            rlist = [rlist]
+        error.collect_exceptions(rlist, self._fname)
+        for r in rlist:
+            yield r
+    
     # asynchronous ordered iterator:
     def _ordered_iter(self):
         """iterator for results *as they arrive*, preserving submission order."""
@@ -664,17 +691,11 @@ class AsyncMapResult(AsyncResult):
             rlist = self.get(0)
         except error.TimeoutError:
             # wait for each result individually
+            evt = Event()
             for child in self._children:
-                ar = AsyncResult(self._client, child, self._fname)
-                rlist = ar.get()
-                try:
-                    for r in rlist:
-                        yield r
-                except TypeError:
-                    # flattened, not a list
-                    # this could get broken by flattened data that returns iterables
-                    # but most calls to map do not expose the `flatten` argument
-                    yield rlist
+                self._wait_for_child(child, evt=evt)
+                for r in self._yield_child_results(child):
+                    yield r
         else:
             # already done
             for r in rlist:
@@ -686,31 +707,14 @@ class AsyncMapResult(AsyncResult):
         try:
             rlist = self.get(0)
         except error.TimeoutError:
-            pending = set(self.msg_ids)
-            while pending:
-                try:
-                    self._client.wait(pending, 1e-3)
-                except error.TimeoutError:
-                    # ignore timeout error, because that only means
-                    # *some* jobs are outstanding
-                    pass
-                # update ready set with those no longer outstanding:
-                ready = pending.difference(self._client.outstanding)
-                # update pending to exclude those that are finished
-                pending = pending.difference(ready)
-                while ready:
-                    msg_id = ready.pop()
-                    child = self._children[self.msg_ids.index(msg_id)]
-                    ar = AsyncResult(self._client, child, self._fname)
-                    rlist = ar.get()
-                    try:
-                        for r in rlist:
-                            yield r
-                    except TypeError:
-                        # flattened, not a list
-                        # this could get broken by flattened data that returns iterables
-                        # but most calls to map do not expose the `flatten` argument
-                        yield rlist
+            queue = Queue()
+            for child in self._children:
+                child.add_done_callback(queue.put)
+            for i in range(len(self)):
+                # use very-large timeout because no-timeout is not interruptible
+                child = queue.get(timeout=_FOREVER)
+                for r in self._yield_child_results(child):
+                    yield r
         else:
             # already done
             for r in rlist:
@@ -760,12 +764,11 @@ class AsyncHubResult(AsyncResult):
                         self.set_result(r)
                 else:
                     results = error.collect_exceptions(results, self._fname)
+                self._success = True
                 self.set_result(self._reconstruct_result(results))
             except Exception as e:
-                self.set_exception(e)
                 self._success = False
-            else:
-                self._success = True
+                self.set_exception(e)
             finally:
                 if self.owner:
                     [self._client.metadata.pop(mid) for mid in self.msg_ids]
