@@ -5,6 +5,8 @@
 
 from __future__ import print_function
 
+import collections
+from concurrent.futures import Future
 import os
 import json
 from threading import Thread, Event, current_thread
@@ -20,7 +22,6 @@ pjoin = os.path.join
 import zmq
 from zmq.eventloop.ioloop import IOLoop
 from zmq.eventloop.zmqstream import ZMQStream
-from tornado.concurrent import Future
 from tornado.gen import multi_future
 
 from traitlets.config.configurable import MultipleInstanceError
@@ -221,6 +222,11 @@ class Metadata(dict):
             raise KeyError(key)
 
 
+def _is_future(f):
+    """light duck-typing check for Futures"""
+    return hasattr(f, 'add_done_callback')
+
+
 class Client(HasTraits):
     """A semi-synchronous client to an IPython parallel cluster
 
@@ -1112,10 +1118,15 @@ class Client(HasTraits):
         True : when all msg_ids are done
         False : timeout reached, some msg_ids still outstanding
         """
+        futures = []
         if jobs is None:
-            theids = self.outstanding
+            if not self.outstanding:
+                return True
+            # make a copy, so that we aren't passing a mutable collection to _futures_for_msgs
+            theids = set(self.outstanding)
         else:
-            if isinstance(jobs, string_types + (int, AsyncResult)):
+            if isinstance(jobs, string_types + (int, AsyncResult)) \
+            or not isinstance(jobs, collections.Iterable):
                 jobs = [jobs]
             theids = set()
             for job in jobs:
@@ -1125,11 +1136,14 @@ class Client(HasTraits):
                 elif isinstance(job, AsyncResult):
                     theids.update(job.msg_ids)
                     continue
+                elif _is_future(job):
+                    futures.append(job)
+                    continue
                 theids.add(job)
-        if not theids.intersection(self.outstanding):
-            return True
+            if not futures and not theids.intersection(self.outstanding):
+                return True
         
-        futures = self._futures_for_msgs(theids)
+        futures.extend(self._futures_for_msgs(theids))
         return self._await_futures(futures, timeout)
     
     def wait_interactive(self, jobs=None, interval=1., timeout=-1.):
@@ -1307,9 +1321,11 @@ class Client(HasTraits):
         worker_args['ip'] = distributed_info['ip']
         worker_args['port'] = distributed_info['port']
         worker_args['nanny'] = nanny
+        # set default ncores=1, since that's how an IPython cluster is typically set up.
+        worker_args.setdefault('ncores', 1)
+        dview.apply_sync(util.become_distributed_worker, **worker_args)
 
         # Finally, return an Executor connected to the Scheduler
-        dview.apply_sync(util.become_distributed_worker, **worker_args)
         executor = distributed.Executor('{ip}:{port}'.format(**distributed_info))
         return executor
 
